Imports Microsoft.IdentityModel.Clients.ActiveDirectory
Imports System.Configuration
Imports System.Net.Http
Imports System.Net.Http.Headers
Imports System.Security.Cryptography.X509Certificates
Imports System.Text
Imports System.Threading.Tasks

Namespace MyConsoleClient
	Friend Class Program
		Shared Sub Main(ByVal args() As String)
			Dim httpClient = New HttpClient
			Dim requestMineType = New MediaTypeWithQualityHeaderValue("application/json")
			httpClient.DefaultRequestHeaders.Accept.Add(requestMineType)

			Dim clientId = ConfigurationManager.AppSettings("ida:ClientID")
			Dim authority = ConfigurationManager.AppSettings("ida:Authority")
			Dim resourceId = ConfigurationManager.AppSettings("ida:ResourceId")
			Dim endpoint = ConfigurationManager.AppSettings("EndpointAddress")
			Dim certName = ConfigurationManager.AppSettings("CertName")

			Dim authContext = New AuthenticationContext(authority)

			Dim clientAssertionCertificate = New ClientAssertionCertificate(clientId, GetX509CertObject(certName))

			'var clientAssertionCertificate = new ClientAssertionCertificate(clientId, LoadX509CertObjectFromFile(@"C:\Temp\MyConsoleClientCert.pfx", "password"));

			Dim authResult = authContext.AcquireTokenAsync(resourceId, clientAssertionCertificate).Result

			httpClient.DefaultRequestHeaders.Authorization = New AuthenticationHeaderValue("Bearer", authResult.AccessToken)

			Dim httpResponse = httpClient.GetAsync(endpoint).Result

			Console.WriteLine(httpResponse.Content.ReadAsStringAsync.Result)
			Console.ReadLine()
		End Sub

		Private Shared Function GetX509CertObject(ByVal certName As String) As X509Certificate2
			Dim store As New X509Store(StoreLocation.CurrentUser)
			Try
				store.Open(OpenFlags.ReadOnly)
				' Place all certificates in an X509Certificate2Collection object.
				Dim certCollection = store.Certificates
				' Find unexpired certificates.
				Dim currentCerts = certCollection.Find(X509FindType.FindByTimeValid, Date.Now, False)
				' From the collection of unexpired certificates, find the ones with the correct name.
				Dim signingCert = currentCerts.Find(X509FindType.FindBySubjectDistinguishedName, certName, False)
				If signingCert.Count = 0 Then
					' No matching certificate found.
					Return Nothing
				End If
				' Return the first certificate in the collection, has the right name and is current.
				Return signingCert(0)
			Finally
				store.Close()
			End Try
		End Function

		Private Shared Function LoadX509CertObjectFromFile(ByVal certFileName As String, ByVal password As String) As X509Certificate2
			Dim cert = New X509Certificate2
			cert.Import(certFileName, password, X509KeyStorageFlags.PersistKeySet)
			Return cert
		End Function
	End Class
End Namespace
